// 002 u_c_delphi_compiler_list
// 19 apr 2018

// -- (C) Felix John COLIBRI 2018
// -- documentation: http://www.felix-colibri.com

// -- detects whether D6 or D7

unit u_c_delphi_compiler_list;
  interface
    uses Classes, Contnrs
      , u_c_environment
      ;

    const
          // -- MUST contain the short name "d6" etc
          k_delphi_5=            'Delphi_d5';
          k_delphi_6=            'Delphi d6';
          k_delphi_Xe2=          'Delphi Xe2';
          k_delphi_Xe2_Windows7= 'Delphi Xe2 Windows7';
          k_delphi_xe7=          'Delphi Xe7';
          k_delphi_Berlin=       'Delphi_Berlin';

    type t_compiler_version= (e_unknown_version, e_d5_version, e_d6_version, e_xe2_version,
            e_xe7_version, e_berlin_version);

         c_delphi_compiler_list= Class; // forward

         c_delphi_compiler=
               // default d6
             class
               private
                 m_c_delphi_parent_compiler_list: c_delphi_compiler_list;

                 m_registry_library_key: String;
                 m_registry_search_path__sub_key: String;
                 m_registry_browsing_path__sub_key: String;

               public
                 m_compiler_version: t_compiler_version;
                 m_delphi_version_name: String;

                 m_registry_delphi_root_dir: String;

                 m_registry_dcp_output_directory: String;
                 m_registry_bpl_output_directory: String;

                 m_dcc32_full_file_name: String;
                 m_package_suffix: String;

                 // -- xe etc
                 m_c_rsvars_environment: c_environment;

                 m_replace_bds: Boolean;

                 constructor create_delphi_compiler(p_registry_string: String;
                    p_compiler_version: t_compiler_version;
                    p_c_delphi_parent_compiler_list: c_delphi_compiler_list);

                 function f_c_self: c_delphi_compiler;

                 procedure analyze_rsvars(p_rsvars_full_file_name: String);
                 procedure initialize_delphi_compiler; Virtual;

                 // -- "unit" : the SearchPath in the DOF for UNITS
                 function f_registry_unit_directories: String; Virtual;
                 // -- "include" : where to look for (*$i xxx *) includes
                 function f_registry_include_directories: String; Virtual;

                 function f_registry_known_package_key: String; Virtual;
                 function f_registry_disabled_package_key: String; Virtual;
                 function f_registry_library_key: String; Virtual;

//                 procedure analyze_rsvars_delphi_environment_variables;

                 procedure display_delphi_compiler;

                 function f_is_correct_path_and_compiler(p_version, p_path: String): Boolean;

                 destructor Destroy; override;
             end; // c_delphi_compiler

         c_delphi_5_compiler=
             class(c_delphi_compiler)
               constructor create_delphi_compiler(p_registry_string: String;
                   p_compiler_version: t_compiler_version;
                   p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
               procedure initialize_delphi_compiler; Override;

               function f_registry_unit_directories: String; Override;
               function f_registry_include_directories: String; Override;
               function f_registry_known_package_key: String; Override;
               function f_registry_disabled_package_key: String; Override;
               function f_registry_library_key: String; Override;
             end; // c_delphi_xe5_compiler

         c_delphi_xe2_compiler=
             class(c_delphi_compiler)
               m_registry_xe2__bds_commondir: String;

               constructor create_delphi_compiler(p_registry_string: String;
                   p_compiler_version: t_compiler_version;
                   p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
               procedure initialize_delphi_compiler; Override;

               function f_registry_unit_directories: String; Override;
               function f_registry_include_directories: String; Override;
               function f_registry_known_package_key: String; Override;
             end;

         c_delphi_xe7_compiler=
             class(c_delphi_compiler)
               m_registry_xe7__bds_commondir: String;

               constructor create_delphi_compiler(p_registry_string: String;
                   p_compiler_version: t_compiler_version;
                   p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
               procedure initialize_delphi_compiler; Override;

               function f_registry_unit_directories: String; Override;
               function f_registry_include_directories: String; Override;
               function f_registry_known_package_key: String; Override;
             end; // c_delphi_xe7_compiler

         c_delphi_Berlin_compiler=
             class(c_delphi_compiler)
               m_registry_berlin__bds_commondir: String;

               constructor create_delphi_compiler(p_registry_string: String;
                   p_compiler_version: t_compiler_version;
                   p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
               procedure initialize_delphi_compiler; Override;

               function f_registry_unit_directories: String; Override;
               function f_registry_include_directories: String; Override;
               function f_registry_known_package_key: String; Override;
             end; // c_delphi_Berlin_compiler

         c_delphi_compiler_list=
             class
                 m_c_compiler_list: TObjectList;
                 m_c_environment: c_environment;
                 m_platform: String;

                 constructor Create;

                 function f_compiler_count: Integer;
                 function f_c_delphi_compiler(p_compiler_index: Integer): c_delphi_compiler;
                 procedure initialize_compiler_list(p_platform: String);

                 procedure display_compiler_list;
                 function f_c_delphi_compiler_version(p_compiler_version: t_compiler_version): c_delphi_compiler;

                 procedure display_environment;

                 destructor Destroy; override;
             end; // c_delphi_compiler_list

    function f_display_compiler_version(p_compiler_version: t_compiler_version): String;

  implementation
    uses SysUtils, TypInfo
      , u_dcc32_registry_helpers
      , u_c_display
      , u_strings


      ;

    function f_display_compiler_version(p_compiler_version: t_compiler_version): String;
      begin
        Result:= GetEnumName(TypeInfo(t_compiler_version), Integer(p_compiler_version));
        Delete(Result, 1, 2);
      end; // f_display_compiler_version

    // -- c_delphi_compiler

    constructor c_delphi_compiler.create_delphi_compiler(p_registry_string: String;
        p_compiler_version: t_compiler_version;
        p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
      begin
        m_compiler_version:= p_compiler_version;
        m_c_delphi_parent_compiler_list:= p_c_delphi_parent_compiler_list;

        m_registry_delphi_root_dir:= ExcludeTrailingPathDelimiter(p_registry_string);
        m_c_rsvars_environment:= c_environment.create_environment(
            'rsvars_environment_'+ f_display_compiler_version(p_compiler_version), Nil);

        initialize_delphi_compiler;
      end; // create_delphi_compiler

    function c_delphi_compiler.f_c_self: c_delphi_compiler;
      begin
        Result:= Self;
      end; // f_c_self

    procedure c_delphi_compiler.analyze_rsvars(p_rsvars_full_file_name: String);

      procedure _add(p_key, p_value: String);
        begin
          with m_c_rsvars_environment do
          begin
            m_c_key_list.Add(p_key);
            m_c_value_list.Add(p_value);
          end;
        end; // _add

      var l_list_index: Integer;
          l_the_line: String;
          l_index: Integer;
          l_key, l_value: String;
          l_BDS_value: String;

      begin // analyze_rsvars
        with tStringList.Create do
        begin
          LoadFromFile(p_rsvars_full_file_name);
          // display(Text);

          For l_list_index:= 0 to Count- 1 do
          begin
            l_the_line:= Strings[l_list_index];
            // display('rsvars '+ l_the_line);

            l_index:= 1;
            f_string_extract_non_blank(l_the_line, l_index);

            l_key:= f_string_extract_until(l_the_line, '=', l_index);
            l_value:= l_the_line;
            System.Delete(l_value, 1, l_index);

            l_key:= Trim(l_key);
            l_value:= Trim(l_value);

            // display(Format('%-15s %s', [l_key, l_value]));

            if l_key<> ''
              then
                with m_c_rsvars_environment do
                begin

                  m_c_key_list.Add(l_key);
                  m_c_value_list.Add(l_value);
                  if SameText(l_key, 'path')
                    then begin
                        fill_path_list(l_value);
                      end;
                  if SameText(l_key, 'BDS')
                    then l_BDS_value:= l_value;
                end;
          end;

          Free;
        end;

        // -- add BDS  bin lib include
        // -- for Berlin BdsUserDir: c:\user\john\doc\embarcadero\studio\18.0
        if Self is c_delphi_xe7_compiler
          then begin
      //        _add('BDSUSERDIR', ''); // l_BDS_value+ '\USERDIR');
              _add('BDSLIB', l_BDS_value+ '\LIB');
            end;

         with m_c_delphi_parent_compiler_list do
           if m_platform<> ''
             then begin
                 m_c_rsvars_environment.set_key_value('platform', m_platform);
      (*
                      display('---');
                      m_c_rsvars_environment.display_environment;
      *)
              end;
      end; // analyze_rsvars

    procedure c_delphi_compiler.initialize_delphi_compiler;
      begin
        m_delphi_version_name:= k_delphi_6; // 'Delphi 6';

        m_registry_library_key:= 'SOFTWARE\Borland\Delphi\6.0\Library';
        m_registry_search_path__sub_key:= 'Search Path';
        m_registry_browsing_path__sub_key:= 'Browsing Path';

        m_registry_dcp_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Borland\Delphi\6.0\Library', 'Package DCP Output');
        if m_registry_dcp_output_directory<> ''
          then m_registry_dcp_output_directory:=
              StringReplace(m_registry_dcp_output_directory, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);

        m_registry_bpl_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Borland\Delphi\6.0\Library', 'Package DPL Output');
        if m_registry_bpl_output_directory<> ''
          then m_registry_bpl_output_directory:=
              StringReplace(m_registry_bpl_output_directory, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
        // display('bpl '+ m_registry_bpl_output_directory);

        m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32.EXE';
        m_package_suffix:= '60';
      end; // initialize_delphi_compiler

    function c_delphi_compiler.f_registry_unit_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);

        if Result<> ''
          then begin
              Result:=
                  // -- StringReplace(string,  old, new)
                  StringReplace(Result, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
            end
          else display('  *** no_registry_search_path')    ;
      end; // f_registry_unit_directories

    function c_delphi_compiler.f_registry_include_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_browsing_path__sub_key);
        if Result<> ''
          then Result:=
              StringReplace(Result, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
      end; // f_registry_include_directories

    function c_delphi_compiler.f_registry_known_package_key: String;
      begin
(*
        k_delphi_6_known_packages_key= // 'HKEY_CURRENT_USER\Software\Borland\Delphi\6.0\Known Packages';
                  'Software\Borland\Delphi\6.0\Known Packages';
*)
        Result:= 'Software\Borland\Delphi\6.0\Known Packages';
      end; // f_registry_known_package_key

    function c_delphi_compiler.f_registry_disabled_package_key: String;
      begin
        Result:= '';
        // HKEY_CURRENT_USER\Software\Borland\Delphi\5.0\Disabled Packages
      end; // f_registry_disabled_package_key

    function c_delphi_compiler.f_registry_library_key: String;
      begin
        Result:= 'Software\Borland\Delphi\6.0\Library\';
      end; // f_registry_disabled_package_key

    function c_delphi_compiler.f_is_correct_path_and_compiler(p_version, p_path: String): Boolean;
      begin
        display('check_ver '+ p_version);
        display('path      '+ p_path);
        display('m_version '+ LowerCase(m_delphi_version_name));

        Result:= (Pos(p_version, LowerCase(m_delphi_version_name))> 0)
          and (Pos(p_version, LowerCase(p_path))> 0)
      end; // f_is_correct_path_and_compiler

    procedure c_delphi_compiler.display_delphi_compiler;

      procedure _display_directory(p_directory: String);
        var l_title: String;

        procedure _check_directory(p_the_directory: String);
          var l_prefix: String;
              l_new_directory: String;
              l_environment_index: Integer;
              l_key, l_value: String;
              // l_change_count: Integer;
          begin
            // -- if some token, remplace

            // display('----------');
            l_new_directory:= p_the_directory;

            with m_c_rsvars_environment, m_c_key_list do
              for l_environment_index:= 0 to COunt- 1 do
              begin
                l_key:= '$('+ m_c_key_list[l_environment_index]+ ')';
                // display(l_key);
                if Pos(LowerCase(l_key), LowerCase(l_new_directory))> 0
                  then begin
                      l_value:= m_c_value_list[l_environment_index];
                      display(Format('      evt %-10s %s', [l_key, l_value]));
                      // -- StringReplace(string,  old, new)
                      // StringReplace(l_new_directory, l_key, l_value, [rfReplaceAll, rfIgnoreCase]);
                      // l_new_directory:= f_replace_all_strings(l_new_directory, l_key, l_value);
                      l_new_directory:= StringReplace(l_new_directory, l_key, l_value, [rfReplaceAll, rfIgnoreCase]);
                    end;
              end;

            if Pos('$(', l_new_directory)> 0
              then display('    **** still $(');

            if DirectoryExists(ExtractFilePath(l_new_directory))
              then l_prefix:= ''
              else l_prefix:= '*** no_dir ';

            display(Format('%-3s %s', [l_title, l_prefix+ l_new_directory]));
            if l_new_directory<> p_the_directory
              then display('      '+ p_the_directory);
            l_title:= '';
          end; // _check_directory

        var l_index: Integer;
            l_directory: String;

        begin // _display_directory
          l_title:= '';
          l_index:= 1;
          l_directory:= '';

          while l_index<= Length(p_directory) do
          begin
            if p_directory[l_index]= ';'
              then begin
                  if m_replace_bds
                    then _check_directory(l_directory)
                    else display('      '+ l_directory);
                  l_directory:= '';
                end
              else l_directory:= l_directory+ p_directory[l_index];
            Inc(l_index);
          end; // while l_index

          if m_replace_bds
            then _check_directory(l_directory)
            else display('      '+ l_directory);
        end; // _display_directory

      procedure _check_and_display_directory(p_title, p_directory: String);
        begin
          if not DirectoryExists(p_directory)
            then display(p_title+ '*** check '+ p_directory)
            else display(p_title+ p_directory);
        end; // _check_and_display_directory

      begin // display_delphi_compiler
        m_replace_bds:= True;
        display_line;
        display('---------------- '+ m_delphi_version_name);

        display_line;
        display('----------');
        m_c_delphi_parent_compiler_list.m_c_environment.display_environment;
        display_line;
        display('----------');
        m_c_rsvars_environment.display_environment;


        display_line;
        display('----------');
        display('  registry_keys ');
        display('    lib_key      '+ m_registry_library_key);
        display('    search_key   '+ m_registry_search_path__sub_key);
        display('    browse_key   '+ m_registry_browsing_path__sub_key);

        display('  dirs');

        _check_and_display_directory('    root ', m_registry_delphi_root_dir);
        _check_and_display_directory('    bpl  ', m_registry_bpl_output_directory);
        _check_and_display_directory('    dcp  ', m_registry_dcp_output_directory);

        display(Format('  unit [%s]', [m_registry_search_path__sub_key]));
        _display_directory(f_registry_unit_directories);
        display(Format('  incl [%s]', [m_registry_browsing_path__sub_key]));
        _display_directory(f_registry_include_directories);

        display('  dcc32');
        if not FileExists(m_dcc32_full_file_name)
          then display('    *** no_file '+ m_dcc32_full_file_name)
          else display('    '+ m_dcc32_full_file_name);
      end; // display_delphi_compiler

    destructor c_delphi_compiler.Destroy;
      begin
        inherited;
      end; // Destroy

    // -- c_delphi_5_compiler

    constructor c_delphi_5_compiler.create_delphi_compiler(
        p_registry_string: String; p_compiler_version: t_compiler_version;
        p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
      begin
        inherited;
      end; // create_delphi_compiler

    procedure c_delphi_5_compiler.initialize_delphi_compiler;
      begin
        m_delphi_version_name:= k_delphi_5; // 'Delphi 6';

        m_registry_library_key:= 'SOFTWARE\Borland\Delphi\5.0\Library';
        m_registry_search_path__sub_key:= 'Search Path';
        m_registry_browsing_path__sub_key:= 'Browsing Path';

        m_registry_dcp_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Borland\Delphi\5.0\Library', 'Package DCP Output');
        if m_registry_dcp_output_directory<> ''
          then m_registry_dcp_output_directory:=
              StringReplace(m_registry_dcp_output_directory, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);

        m_registry_bpl_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Borland\Delphi\5.0\Library', 'Package DPL Output');
        if m_registry_bpl_output_directory<> ''
          then m_registry_bpl_output_directory:=
              StringReplace(m_registry_bpl_output_directory, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
        // display('bpl '+ m_registry_bpl_output_directory);

        m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32.EXE';
        m_package_suffix:= '50';
      end; // initialize_delphi_compiler

    function c_delphi_5_compiler.f_registry_unit_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);
        if Result<> ''
          then Result:=
              // -- StringReplace(string,  old, new)
              StringReplace(Result, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase])
          else display('  *** no_registry_search_path')    ;
      end; // f_registry_unit_directories

    function c_delphi_5_compiler.f_registry_include_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);
        if Result<> ''
          then Result:=
              // -- StringReplace(string,  old, new)
              StringReplace(Result, '$(DELPHI)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase])
          else display('  *** no_registry_search_path')    ;
      end; // f_registry_include_directories

    function c_delphi_5_compiler.f_registry_known_package_key: String;
      begin
        Result:= 'Software\Borland\Delphi\5.0\Known Packages';
      end; // f_registry_known_package_key

    function c_delphi_5_compiler.f_registry_disabled_package_key: String;
      begin
        Result:= 'Software\Borland\Delphi\5.0\Disabled Packages';
      end; // f_registry_disabled_package_key

    function c_delphi_5_compiler.f_registry_library_key: String;
      begin
        Result:= 'Software\Borland\Delphi\5.0\Library\';
      end; // f_registry_disabled_package_key

    // -- c_delphi_xe2_compiler

    constructor c_delphi_xe2_compiler.create_delphi_compiler(p_registry_string: String;
        p_compiler_version: t_compiler_version;
        p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
      begin
        Inherited;
      end; // create_delphi_compiler

    procedure c_delphi_xe2_compiler.initialize_delphi_compiler;
      begin
        m_delphi_version_name:= k_delphi_Xe2;

        // -- TODO : find by registry
        m_registry_xe2__bds_commondir:= 'C:\Documents and Settings\All Users\Documents\RAD Studio\9.0';

        m_registry_library_key:= 'SOFTWARE\Embarcadero\BDS\9.0\Library\Win32';
        m_registry_search_path__sub_key:= 'Search Path';
        m_registry_browsing_path__sub_key:= 'Browsing Path';

        m_registry_dcp_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\9.0\Library\Win32', 'Package DCP Output');
        if m_registry_dcp_output_directory<> ''
          then m_registry_dcp_output_directory:=
              StringReplace(m_registry_dcp_output_directory, '$(BDSCOMMONDIR)', m_registry_xe2__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        m_registry_bpl_output_directory:= f_read_registry_string(
            k_HKEY_current_user,'SOFTWARE\Embarcadero\BDS\9.0\Library\Win32', 'Package DPL Output');
        if m_registry_bpl_output_directory<> ''
          then m_registry_bpl_output_directory:=
              StringReplace(m_registry_bpl_output_directory, '$(BDSCOMMONDIR)', m_registry_xe2__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        // m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32.EXE';
        m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32_xe2_ds.EXE';

        m_package_suffix:= '160';
      end; // initialize_delphi_compiler

    function c_delphi_xe2_compiler.f_registry_unit_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);
      end; // f_registry_unit_directories

    function c_delphi_xe2_compiler.f_registry_include_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_browsing_path__sub_key);
        if Result<> ''
          then Result:= StringReplace(
              Result, '$(BDS)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
      end; // f_registry_include_directories

    function c_delphi_xe2_compiler.f_registry_known_package_key: String;
      begin
        Result:= 'Software\Embarcadero\BDS\9.0\Known Packages';
      end; // f_registry_known_package_key

    // -- c_delphi_xe7_compiler

    constructor c_delphi_xe7_compiler.create_delphi_compiler(p_registry_string: String;
        p_compiler_version: t_compiler_version;
        p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
      begin
        Inherited;
      end; // create_delphi_compiler

    procedure c_delphi_xe7_compiler.initialize_delphi_compiler;
      var l_rsvars_full_file_name: String;
      begin
        l_rsvars_full_file_name:= m_registry_delphi_root_dir+ '\bin\rsvars.bat';
        if not FileExists(l_rsvars_full_file_name)
          then display_bug_stop('not_found '+ l_rsvars_full_file_name)
          else analyze_rsvars(l_rsvars_full_file_name);

        m_delphi_version_name:= k_delphi_xe7;

        m_registry_xe7__bds_commondir:=
            m_c_rsvars_environment.f_key_value('BDSCOMMONDIR');

        m_registry_library_key:= 'SOFTWARE\Embarcadero\BDS\15.0\Library\Win32';
        m_registry_search_path__sub_key:= 'Search Path';
        m_registry_browsing_path__sub_key:= 'Browsing Path';

        m_registry_dcp_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\15.0\Library\Win32', 'Package DCP Output');
        if m_registry_dcp_output_directory<> ''
          then m_registry_dcp_output_directory:=
              StringReplace(m_registry_dcp_output_directory, '$(BDSCOMMONDIR)', m_registry_xe7__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        m_registry_bpl_output_directory:= f_read_registry_string(
            k_HKEY_current_user,'SOFTWARE\Embarcadero\BDS\15.0\Library\Win32', 'Package DPL Output');
        if m_registry_bpl_output_directory<> ''
          then m_registry_bpl_output_directory:=
              StringReplace(m_registry_bpl_output_directory, '$(BDSCOMMONDIR)', m_registry_xe7__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32_xe7_ds.EXE';

        // dsds ?
        m_package_suffix:= '160';
      end; // initialize_delphi_compiler

    function c_delphi_xe7_compiler.f_registry_unit_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);
      end; // f_registry_unit_directories

    function c_delphi_xe7_compiler.f_registry_include_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_browsing_path__sub_key);
        if Result<> ''
          then Result:= StringReplace(
              Result, '$(BDS)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
      end; // f_registry_include_directories

    function c_delphi_xe7_compiler.f_registry_known_package_key: String;
      begin
        Result:= 'Software\Embarcadero\BDS\15.0\Known Packages';
      end; // f_registry_known_package_key

    // -- c_delphi_Berlin_compiler

    constructor c_delphi_Berlin_compiler.create_delphi_compiler(p_registry_string: String;
        p_compiler_version: t_compiler_version;
        p_c_delphi_parent_compiler_list: c_delphi_compiler_list);
      begin
        Inherited;
      end; // create_delphi_compiler

    procedure c_delphi_Berlin_compiler.initialize_delphi_compiler;
      var l_rsvars_full_file_name: String;
      begin
        l_rsvars_full_file_name:= m_registry_delphi_root_dir+ '\bin\rsvars.bat';
        if not FileExists(l_rsvars_full_file_name)
          then display_bug_stop('not_found '+ l_rsvars_full_file_name)
          else analyze_rsvars(l_rsvars_full_file_name);

        m_delphi_version_name:= k_delphi_Berlin;

        m_registry_berlin__bds_commondir:=
            m_c_rsvars_environment.f_key_value('BDSCOMMONDIR');

        m_registry_library_key:= 'SOFTWARE\Embarcadero\BDS\18.0\Library\Win32';
        m_registry_search_path__sub_key:= 'Search Path';
        m_registry_browsing_path__sub_key:= 'Browsing Path';

        m_registry_dcp_output_directory:= f_read_registry_string(
            k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\18.0\Library\Win32', 'Package DCP Output');
        if m_registry_dcp_output_directory<> ''
          then m_registry_dcp_output_directory:=
              StringReplace(m_registry_dcp_output_directory, '$(BDSCOMMONDIR)',
                  m_registry_Berlin__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        m_registry_bpl_output_directory:= f_read_registry_string(
            k_HKEY_current_user,'SOFTWARE\Embarcadero\BDS\18.0\Library\Win32', 'Package DPL Output');
        if m_registry_bpl_output_directory<> ''
          then m_registry_bpl_output_directory:=
              StringReplace(m_registry_bpl_output_directory, '$(BDSCOMMONDIR)',
                  m_registry_Berlin__bds_commondir, [rfReplaceAll, rfIgnoreCase]);

        m_dcc32_full_file_name:= m_registry_delphi_root_dir+ '\Bin\DCC32.EXE';

        // dsds ?
        m_package_suffix:= '160';
      end; // initialize_delphi_compiler

    function c_delphi_Berlin_compiler.f_registry_unit_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_search_path__sub_key);
      end; // f_registry_unit_directories

    function c_delphi_Berlin_compiler.f_registry_include_directories: String;
      begin
        Result:= f_read_registry_string(
            k_HKEY_current_user, m_registry_library_key, m_registry_browsing_path__sub_key);
        if Result<> ''
          then Result:= StringReplace(
              Result, '$(BDS)', m_registry_delphi_root_dir, [rfReplaceAll, rfIgnoreCase]);
      end; // f_registry_include_directories

    function c_delphi_Berlin_compiler.f_registry_known_package_key: String;
      begin
        Result:= 'Software\Embarcadero\BDS\15.0\Known Packages';
      end; // f_registry_known_package_key

    // -- c_delphi_compiler_list

    constructor c_delphi_compiler_list.Create;
      begin
        m_c_compiler_list:= TObjectList.Create;

        m_c_environment:= c_environment.create_environment('globa_environment', Nil);
        with m_c_environment do
        begin
          analyze_environment;
          // display_environment;
        end;
      end; // Create

    function c_delphi_compiler_list.f_compiler_count: Integer;
      begin
        Result:= m_c_compiler_list.Count;
      end; // f_compiler_count

    function c_delphi_compiler_list.f_c_delphi_compiler(p_compiler_index: Integer): c_delphi_compiler;
      begin
        Result:= c_delphi_compiler(m_c_compiler_list[p_compiler_index]);
      end; // f_c_delphi_compiler

    procedure c_delphi_compiler_list.initialize_compiler_list(p_platform: String);
        // -- compiler search order is descending, i.e., most recent one is the first ..
      var l_c_delphi_compiler: c_delphi_compiler;
          l_registry_string: String;
      begin
        display_line;
        display('> initialize_compiler_list '+ p_platform);

        m_platform:= p_platform;

        // -- Delphi 5 support :
        if f_simple_registry_readstring(k_HKEY_current_user, 'Software\Borland\Delphi\5.0\ActiveX Controls',
            'UnitDirName', l_registry_string)
          then begin
              // HKEY_CURRENT_USER\Software\Borland\Delphi\5.0\ActiveX Controls
              // UnitDirName C:\Program Files (x86)\Borland\Delphi5\Imports
              l_registry_string:= f_remove_end_if_end_is_equal_to(l_registry_string, '\Imports');
              display('found_registry_d5 '+ l_registry_string);
              l_c_delphi_compiler:= c_delphi_5_compiler.create_delphi_compiler(
                  l_registry_string, e_d5_version, Self);
              m_c_compiler_list.Add(l_c_delphi_compiler);
            end;

        // -- Delphi 6 support :
        if f_simple_registry_readstring(k_HKEY_current_user, 'SOFTWARE\Borland\Delphi\6.0', 'RootDir', l_registry_string)
          then begin
              display('found_registry_d6 '+ l_registry_string);
              l_c_delphi_compiler:= c_delphi_compiler.create_delphi_compiler(
                  l_registry_string, e_d6_version, Self);
              m_c_compiler_list.Add(l_c_delphi_compiler);
            end;

        // -- Delphi xe2 support :
        if f_simple_registry_readstring(k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\9.0', 'RootDir', l_registry_string)
          then begin
              display('found_registry_xe2');
              l_c_delphi_compiler:= c_delphi_xe2_compiler.create_delphi_compiler(
                  l_registry_string, e_xe2_version, Self);
              m_c_compiler_list.Add(l_c_delphi_compiler);
            end;

        // -- Delphi Xe7 support :
        if f_simple_registry_readstring(k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\15.0', 'RootDir', l_registry_string)
          then begin
              display('found_registry_xe7');
              l_c_delphi_compiler:= c_delphi_xe7_compiler.create_delphi_compiler(
                  l_registry_string, e_xe7_version, Self);
              m_c_compiler_list.Add(l_c_delphi_compiler);
            end;

        // -- Delphi Berlin support :
        if f_simple_registry_readstring(k_HKEY_current_user, 'SOFTWARE\Embarcadero\BDS\18.0', 'RootDir', l_registry_string)
          then begin
              display('found_registry_Berlin');
              l_c_delphi_compiler:= c_delphi_Berlin_compiler.create_delphi_compiler(
                  l_registry_string, e_berlin_version, Self);
              m_c_compiler_list.Add(l_c_delphi_compiler);
            end;

        display('< initialize_compiler_list');
      end; // initialize_compiler_list

    procedure c_delphi_compiler_list.display_compiler_list;
      var l_compiler_index: Integer;
      begin
        with m_c_compiler_list do
          for l_compiler_index:= 0 to f_compiler_count- 1 do
            with f_c_delphi_compiler(l_compiler_index) do
            begin
              display(Format('%2d %-11s %s', [l_compiler_index,
                  // f_compiler_item_index(m_compiler_version),
                  f_display_compiler_version(m_compiler_version), m_delphi_version_name]));
            end;
      end; // display_compiler_list

    function c_delphi_compiler_list.f_c_delphi_compiler_version(p_compiler_version: t_compiler_version): c_delphi_compiler;
      var l_compiler_index: Integer;
      begin
        Result:= Nil;
        with m_c_compiler_list do
          for l_compiler_index:= 0 to f_compiler_count- 1 do
            with f_c_delphi_compiler(l_compiler_index) do
              if m_compiler_version= p_compiler_version
                then Result:= f_c_self;
      end; // f_c_delphi_compiler_version

    procedure c_delphi_compiler_list.display_environment;
      begin
      end;

    destructor c_delphi_compiler_list.Destroy;
      begin
        m_c_compiler_list.Free;

        inherited;
      end; // Destroy

  end.    











