// 009 u_c_environment
// 19 apr 2018

// -- (C) Felix John COLIBRI 2018
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_environment;
  interface
    uses Classes, u_c_basic_object;

    type c_environment=
            Class(c_basic_object)
              m_size: Integer;
              m_oa_environment: array of char;
              m_c_key_list, m_c_value_list: tStringList;
              m_c_path_list: tStringList;

              constructor create_environment(p_name: String; p_c_strings: tStrings);
              procedure analyze_environment;
              procedure fill_path_list(p_pathes: String);
              procedure display_environment;
              function f_key_value(p_key: String): String;
              procedure set_key_value(p_key, p_value: String);
              Destructor Destroy; Override;
            end; // c_environment

    function f_write_environment_key_value(p_key, p_value: String): Boolean;
    function f_get_environment_value(p_key: pChar): String;
    function f_get_environment_value_2(const VarName: string): string;
    function f_get_all_environment_string: String;

  implementation
    uses Windows
        , SysUtils // StrPas
        , u_c_display
        , u_strings
        ;

    function f_write_environment_key_value(p_key, p_value: String): Boolean;
      begin
        if (p_key<> '') and (p_value<> '')
          then Result:= SetEnvironmentVariable(pChar(p_key), pChar(p_value))
          else Result:= False;
      end; // write_environment_key_value

    function f_get_environment_value(p_key: pChar): String;
      const k_max= 255;
      var l_environment_value: String[k_max];
          l_size: Integer;
      begin
        display_bug_stop('use f_get_environment_value_2');
        // -- must use k_max, but the true size can be larger !!
        l_size:= GetEnvironmentVariable(p_key, @ l_environment_value[1], k_max);
        l_environment_value[0]:= chr(l_size);
        Result:= l_environment_value;
      end; // f_get_environment_value

    function f_get_environment_value_2(const VarName: string): string;
      var BufSize: Integer;  // buffer size required for value
      begin
        // Get required buffer size (inc. terminal #0)
        BufSize := Windows.GetEnvironmentVariable(PChar(VarName) , nil, 0);
        if BufSize > 0
          then begin
              // Read env var value into result string
              SetLength(Result, BufSize - 1);
              GetEnvironmentVariable(PChar(VarName), PChar(Result), BufSize);
            end
          else
            // No such environment variable
            Result := '';
      end; // f_get_environment_value_2

    function f_get_all_environment_string: String;
        // -- get all the environment variables
      var l_pchar: PChar;
          l_running_pchar: pChar;
          l_environment_line: ShortString;
      begin
        l_pchar:= GetEnvironmentStrings;

        Result:= '';
        l_running_pchar:= l_pchar;

        while l_running_pchar^<> #0 do
        begin
          l_environment_line:= StrPas(l_running_pchar);
          Result:= Result+ l_environment_line+ chr(13)+chr(10);
          Inc(l_running_pchar, StrLen(l_running_pchar)+ 1)
        end; // while

        FreeEnvironmentStrings(l_pchar);
      end; // f_get_all_environment_string

    // -- c_environment

    constructor c_environment.create_environment(p_name: String; p_c_strings: tStrings);
        // -- create a block of xxx0yyy0zzz00
      var l_line: Integer;
          l_position: Integer;
      begin
        Inherited create_basic_object(p_name);
        
        m_c_path_list:= tStringList.Create;

        m_size:= 0;
        if p_c_strings<> Nil
          then
            with p_c_strings do
            begin
              for l_line:= 0 to Count- 1 do
                Inc(m_size, Length(Strings[l_line])+ 1);
              Inc(m_size);

              SetLength(m_oa_environment, m_size);
              l_position:= 0;
              for l_line:= 0 to Count- 1 do
              begin
                System.Move(p_c_strings[l_line][1], m_oa_environment[l_position], Length(Strings[l_line]));
                Inc(l_position, Length(Strings[l_line]));
                m_oa_environment[l_position]:= Chr(0);
                Inc(l_position);
              end;

              // -- add another 0 at the end
              m_oa_environment[m_size- 1]:= Chr(0);
            end;

        m_c_key_list:= tStringList.Create;
        m_c_key_list.CaseSensitive:= False;
        m_c_value_list:= tStringList.Create;
      end; // c_environment

    procedure c_environment.fill_path_list(p_pathes: String);
      var l_path_index: Integer;
          l_path: String;
      begin
        l_path_index:= 1;

        while l_path_index<= Length(p_pathes) do
        begin
          l_path:= f_string_extract_until(p_pathes, ';', l_path_index);
          // display(f_spaces(28)+ l_path);
          m_c_path_list.Add(l_path);
          Inc(l_path_index);
        end;
      end; // fill_path_list

    procedure c_environment.analyze_environment;

      function _f_get_environment_value(const VarName: string): string;
        var BufSize: Integer;  // buffer size required for value
        begin
          // Get required buffer size (inc. terminal #0)
          BufSize := GetEnvironmentVariable(PChar(VarName), nil, 0);
          if BufSize > 0
            then begin
                // Read env var value into result string
                SetLength(Result, BufSize - 1);
                GetEnvironmentVariable(PChar(VarName), PChar(Result), BufSize);
              end
            else // No such environment variable
              Result := '';
        end; // _f_get_environment_value

      var l_pchar: PChar;
          l_running_pchar: pChar;
          l_environment_line: ShortString;
          l_index, l_length: Integer;
          l_key, l_value: String;

      begin // analyze_environment
        l_pchar:= GetEnvironmentStrings;
        // display(l_pchar);

        l_running_pchar:= l_pchar;

        while l_running_pchar^<> #0 do
        begin
          l_environment_line:= StrPas(l_running_pchar);

          // display(l_environment_line);
          l_index:= 1; l_length:= Length(l_environment_line);
          l_key:= '';
          while (l_index<= l_length) and (l_environment_line[l_index]<> '=') do
          begin
            l_key:= l_key+ l_environment_line[l_index];
            Inc(l_index);
          end;

          l_key:= Trim(l_key);

          if SameText(l_key, 'path')
            then begin
                l_value:= _f_get_environment_value('path');
                fill_path_list(l_value);
              end
            else l_value:= l_environment_line;

          System.Delete(l_value, 1, l_index);
          l_value:= Trim(l_value);

          if l_key<> ''
            then begin
                // display(' >'+ l_key+ ' < >'+ l_value+ '<');
                m_c_key_list.Add(l_key);
                m_c_value_list.Add(l_value);
              end;

          Inc(l_running_pchar, StrLen(l_running_pchar)+ 1)
        end; // while

        FreeEnvironmentStrings(l_pchar);
      end; // analyze_environment

    procedure c_environment.display_environment;
      var l_index: Integer;
          l_key: String;
          l_path_index: integer;
      begin
        display_line;
        display(m_name);
        with m_c_key_list do
          for l_index:= 0 to Count- 1 do
          begin
            l_key:= Strings[l_index];
            if SameText(l_key, 'path')
              then begin
                  display('Path');

                  with m_c_path_list do
                    for l_path_index:= 0 to Count- 1 do
                      display(f_spaces(28)+ Strings[l_path_index]);

                end
              else display(Format('%-25s %s', [ l_key, m_c_value_list[l_index] ]));
          end;
      end; // display_environment

    function c_environment.f_key_value(p_key: String): String;
      var l_index_of: Integer;
          l_index: Integer;
      begin
        l_index_of:= m_c_key_list.IndexOf(p_key);

        if l_index_of< 0
          then begin
              display('--->');
              display(m_c_key_list.Text);
              // display_environment;
              display('---<');
              with m_c_key_list do
                for l_index:= 0 to COunt- 1 do
                  display(' >'+ Strings[l_index]+ '< >'+ p_key+ '<');
              display_bug_stop('envir_key_not_found '+ p_key);
              Result:= '';
            end
          else Result:= m_c_value_list[l_index_of];
      end; // f_key_value

    procedure c_environment.set_key_value(p_key, p_value: String);
      var l_index_of: Integer;
      begin
        l_index_of:= m_c_key_list.IndexOf(p_key);

        if l_index_of< 0
          then display_bug_stop('key_not_found '+ p_key)
          else m_c_value_list[l_index_of]:= p_value;
      end; // set_key_value

    Destructor c_environment.Destroy;
      begin
        m_c_key_list.Free;
        m_c_value_list.Free;
        FreeAndNil(m_c_path_list);

        InHerited;
      end; // Destroy

    begin // u_c_yyy
    end. // u_c_yyy

