// 002 u_dcc32_registry_helpers
// 19 apr 2018

// -- (C) Felix John COLIBRI 2018
// -- documentation: http://www.felix-colibri.com

unit u_dcc32_registry_helpers;
  interface
    uses Windows, Registry, Classes; //, SysUtils

    const // k_hkey_local_machine= HKEY_LOCAL_MACHINE;
          k_HKEY_current_user= HKEY_CURRENT_USER;
          k_HKEY_users= HKEY_USERS; 

    function f_simple_registry_readstring(RootKey: HKEY; const Path, Name: String; var Value:
          String; bForce: Boolean= false): Boolean;
          {
          HLReg stands for High Level Registry.
          That is slow but easy to use.
          Suitable for light registry operations where speed is not important.
          }

    function HLRegWriteString(RootKey: HKEY; const Path, Name: String; const
          Value: String): Boolean;
      { RegReadBinary note:
      If BufferSize is set 0 while calling, Buffer of required size allocated automatically.
      If returns true and above is the case, Caller is responsible from releasing the memory by FreeMem. }

    function HLRegReadBinary(RootKey: HKEY; const Path, Name: String; var Buffer:
          Pointer; var BufferSize: Integer; bForce: Boolean= false): Boolean;
    function HLRegReadBinaryFixed(RootKey: HKEY; const Path, Name: String; Buffer:
          Pointer; BufferSize: Integer; bForce: Boolean= false): Boolean;
    function HLRegWriteBinary(RootKey: HKEY; const Path, Name: String; Buffer:
          Pointer; BufferSize: Integer): Boolean;
    function HLRegReadBinaryString(RootKey: HKEY; const Path, Name: String; var
          Value: String; bForce: Boolean= false): Boolean;
    function HLRegWriteBinaryString(RootKey: HKEY; const Path, Name: String;
          const Value: String): Boolean;
    function HLRegReadInteger(RootKey: HKEY; const Path, Name: String; var Value:
          Integer; bForce: Boolean= false): Boolean;
    function HLRegWriteInteger(RootKey: HKEY; const Path, Name: String; Value:
          Integer): Boolean;
    function HLRegReadBoolean(RootKey: HKEY; const Path, Name: String; var Value:
          Boolean; bForce: Boolean= false): Boolean;
    function HLRegWriteBoolean(RootKey: HKEY; const Path, Name: String; Value:
          Boolean): Boolean;
    function HLRegReadDateTime(RootKey: HKEY; const Path, Name: String; var Value:
          TDateTime; bForce: Boolean= false): Boolean;
    function HLRegWriteDateTime(RootKey: HKEY; const Path, Name: String; Value:
          TDateTime): Boolean;
    function HLRegKeyExists(RootKey: HKEY; const Path: String; const Name: String= ''
          ): Boolean;
    function HLRegCreateKey(RootKey: HKEY; const Path: String; const Name: String= ''
          ): Boolean;
    function HLRegDeleteKey(RootKey: HKEY; const Path: String; const Key: String= '')
          : Boolean;
    function HLRegMoveKey(RootKey: HKEY; const OldPath, NewPath: String): Boolean;
    function HLRegDeleteValue(RootKey: HKEY; const Path, Name: String): Boolean;
    function HLRegGetKeyNames(RootKey: HKEY; const Path: String; Strings:
          TStrings): Boolean;
    function HLRegGetValueNames(RootKey: HKEY; const Path: String; Strings:
          TStrings): Boolean;
    function f_read_registry_string(RootKey: HKEY; const Path, Name: String): String;

  implementation
    uses u_c_display;

    function f_read_registry_string(RootKey: HKEY; const Path, Name: String): String;
      var l_registry: TRegistry;
      begin
        Result:= '';
        l_registry:= TRegistry.Create;
        try
          l_registry.RootKey:= RootKey;
          if l_registry.OpenKeyReadOnly(Path)
            then
              if l_registry.ValueExists(Name)
                then
                  begin
                    Result:= l_registry.ReadString(Name);
                  end;
        except
          display('*** pb_OpenRegKey RootKey '+ Path);
        end;

        l_registry.Free; // Calls CloseKey
      end; // f_read_registry_string

    function f_simple_registry_readstring(RootKey: HKEY; const Path, Name: String;
        var Value: String;
        bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    Value:= Reg.ReadString(Name);
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey

        if (not Result) and bForce
          then Result:= HLRegWriteString(RootKey, Path, Name, Value);
      end; // f_simple_registry_readstring

    function HLRegWriteString(RootKey: HKEY; const Path, Name: String; const
          Value: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, True)
            then
              begin
                Reg.WriteString(Name, Value);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegWriteString   

    function HLRegReadBinary(RootKey: HKEY; const Path, Name: String; var Buffer:
          Pointer; var BufferSize: Integer; bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
          RegData: TRegDataInfo;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    if BufferSize= 0
                      then
                        begin
                          Reg.GetDataInfo(Name, RegData);
                          BufferSize:= RegData.DataSize;
                          GetMem(Buffer, BufferSize);
                        end;
                    BufferSize:= Reg.ReadBinaryData(Name, Buffer^, BufferSize);
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
        if (not Result) and bForce
          then Result:= HLRegWriteBinary(RootKey, Path, Name, Buffer, BufferSize);
      end; // HLRegReadBinary   

    function HLRegReadBinaryFixed(RootKey: HKEY; const Path, Name: String; Buffer:
          Pointer; BufferSize: Integer; bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    FillChar(Buffer^, BufferSize, 0);
                    Reg.ReadBinaryData(Name, Buffer^, BufferSize);
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
        if (not Result) and bForce
          then Result:= HLRegWriteBinary(RootKey, Path, Name, Buffer, BufferSize);
      end; // HLRegReadBinaryFixed   

    function HLRegWriteBinary(RootKey: HKEY; const Path, Name: String; Buffer:
          Pointer; BufferSize: Integer): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, True)
            then
              begin
                Reg.WriteBinaryData(Name, Buffer^, BufferSize);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegWriteBinary   

    function HLRegReadBinaryString(RootKey: HKEY; const Path, Name: String; var
          Value: String; bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
          RegData: TRegDataInfo;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    Reg.GetDataInfo(Name, RegData);
                    if RegData.DataSize= 0
                      then Value:= ''
                      else
                        begin
                          SetLength(Value, RegData.DataSize);
                          Reg.ReadBinaryData(Name, Value[1], RegData.DataSize);
                        end;
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
        if (not Result) and bForce
          then Result:= HLRegWriteBinaryString(RootKey, Path, Name, Value);
      end; // HLRegReadBinaryString   

    function HLRegWriteBinaryString(RootKey: HKEY; const Path, Name: String;
          const Value: String): Boolean;
      var Reg: TRegistry;
          tmp: String;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, True)
            then
              begin
                tmp:= Value;
                if tmp= ''
                  then Reg.WriteBinaryData(Name, tmp, 0)// Value is a dummy parameter here

                  else Reg.WriteBinaryData(Name, tmp[1], Length(tmp));
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegWriteBinaryString   

    function HLRegReadInteger(RootKey: HKEY; const Path, Name: String; var Value:
          Integer; bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    Value:= Reg.ReadInteger(Name);
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
        if (not Result) and bForce
          then Result:= HLRegWriteInteger(RootKey, Path, Name, Value);
      end; // HLRegReadInteger   

    function HLRegWriteInteger(RootKey: HKEY; const Path, Name: String; Value:
          Integer): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, True)
            then
              begin
                Reg.WriteInteger(Name, Value);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegWriteInteger   

    function HLRegReadBoolean(RootKey: HKEY; const Path, Name: String; var Value:
          Boolean; bForce: Boolean= false): Boolean;
      var tmpint: Integer;
      begin
        if Value
          then tmpint:= 1
          else tmpint:= 0;
        Result:= HLRegReadInteger(RootKey, Path, Name, tmpint, bForce);
        Value:= tmpint<> 0;
      end; // HLRegReadBoolean   

    function HLRegWriteBoolean(RootKey: HKEY; const Path, Name: String; Value:
          Boolean): Boolean;
      var tmpint: Integer;
      begin
        if Value
          then tmpint:= 1
          else tmpint:= 0;
        Result:= HLRegWriteInteger(RootKey, Path, Name, tmpint);
      end; // HLRegWriteBoolean   

    function HLRegReadDateTime(RootKey: HKEY; const Path, Name: String; var Value:
          TDateTime; bForce: Boolean= false): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              if Reg.ValueExists(Name)
                then
                  begin
                    Value:= Reg.ReadDateTime(Name);
                    Result:= true;
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
        if (not Result) and bForce
          then Result:= HLRegWriteDateTime(RootKey, Path, Name, Value);
      end; // HLRegReadDateTime   

    function HLRegWriteDateTime(RootKey: HKEY; const Path, Name: String; Value:
          TDateTime): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, True)
            then
              begin
                Reg.WriteDateTime(Name, Value);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegWriteDateTime   

    function HLRegKeyExists(RootKey: HKEY; const Path, Name: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              begin
                if Length(Name)= 0
                  then Result:= true
                  else Result:= Reg.KeyExists(Name);
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegKeyExists   

    function HLRegCreateKey(RootKey: HKEY; const Path, Name: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, true)
            then
              begin
                if Length(Name)= 0
                  then Result:= true
                  else Result:= Reg.CreateKey(Name);
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegCreateKey   

    function HLRegDeleteKey(RootKey: HKEY; const Path, Key: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Length(Key)= 0
            then Result:= Reg.DeleteKey(Path)
            else
              if Reg.OpenKey(Path, false)
                then
                  begin
                    Result:= Reg.DeleteKey(Key);
                  end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegDeleteKey   

    function HLRegMoveKey(RootKey: HKEY; const OldPath, NewPath: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          Reg.MoveKey(OldPath, NewPath, true);
          Result:= true;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegMoveKey   

    function HLRegDeleteValue(RootKey: HKEY; const Path, Name: String): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKey(Path, false)
            then
              begin
                Result:= Reg.DeleteValue(Name);
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegDeleteValue   

    function HLRegGetKeyNames(RootKey: HKEY; const Path: String; Strings:
          TStrings): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              begin
                Reg.GetKeyNames(Strings);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegGetKeyNames   

    function HLRegGetValueNames(RootKey: HKEY; const Path: String; Strings:
          TStrings): Boolean;
      var Reg: TRegistry;
      begin
        Result:= false;
        Reg:= TRegistry.Create;
        try
          Reg.RootKey:= RootKey;
          if Reg.OpenKeyReadOnly(Path)
            then
              begin
                Reg.GetValueNames(Strings);
                Result:= true;
              end;
        except
        end;
        Reg.Free; // Calls CloseKey
      end; // HLRegGetValueNames

    end.
